using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using UnityEngine;

namespace RealTime.Pandemic
{
    class ContactManager
    {
        public static ContactManager Instance { get; } = new ContactManager();

        private System.Random random = new System.Random(1337);

        private Dictionary<uint, Dictionary<uint, DateTime>> contacts = new Dictionary<uint, Dictionary<uint, DateTime>>();

        private HashSet<uint> citizensUsingContactTracingBuilding = new HashSet<uint>();
        private HashSet<uint> citizensNotUsingContactTracingBuilding = new HashSet<uint>();

        private HashSet<uint> citizensUsingContactTracingApp = new HashSet<uint>();
        private HashSet<uint> citizensNotUsingContactTracingApp = new HashSet<uint>();

        private Config.RealTimeConfig config;

        public void Init(Config.RealTimeConfig config)
        {
            this.config = config;
        }

        private bool CitizenUsesContactTracing(uint citizenId, bool inBuilding)
        {
            if (!citizensUsingContactTracingApp.Contains(citizenId) && !citizensNotUsingContactTracingApp.Contains(citizenId))
            {
                if (random.NextDouble() < config.AppBasedContactTracingProbability / 100.0)
                {
                    citizensUsingContactTracingApp.Add(citizenId);
                }
                else
                {
                    citizensNotUsingContactTracingApp.Add(citizenId);
                }
            }

            if (inBuilding)
            {
                if (!citizensUsingContactTracingBuilding.Contains(citizenId) && !citizensNotUsingContactTracingBuilding.Contains(citizenId))
                {
                    if (random.NextDouble() < config.BuildingContactTracingProbability / 100.0)
                    {
                        citizensUsingContactTracingBuilding.Add(citizenId);
                    } else
                    {
                        citizensNotUsingContactTracingBuilding.Add(citizenId);
                    }
                }
            }
            return (citizensUsingContactTracingBuilding.Contains(citizenId) && inBuilding) || citizensUsingContactTracingApp.Contains(citizenId);
        }

        public void AddContact(uint citizenId, uint contactId, bool inBuilding, DateTime currentTime)
        {
            if (CitizenUsesContactTracing(citizenId, inBuilding) && CitizenUsesContactTracing(contactId, inBuilding)) {
                if (!contacts.ContainsKey(citizenId))
                {
                    contacts.Add(citizenId, new Dictionary<uint, DateTime>());
                }
                contacts[citizenId][contactId] = currentTime;
            }
        }

        public Dictionary<uint, DateTime> GetContactsForCitizen(uint citizenId)
        {
            if (contacts.ContainsKey(citizenId))
            {
                return contacts[citizenId];
            }
            return null;
        }
    }
}
